use XML::Simple;

require "/var/lib/asterisk/agi-bin/rights.pl";

sub tollrest_debug {
	my $line = shift;
	print "$line\n";
	return;

	open (DEB, ">>/tmp/tollrest.log");
	print DEB $line;
	close (DEB);
}

test(undef, type => 'extension', content => 7026);

sub test
{
	# get our AGI object. also, all of the arguments to the AGI() are
	# passed in @_ so %arguments will equal (test => 123, cid => 5551212)
	my ($AGI, %arguments) = @_;

	# Get their intended dialing string.
	my $type    = $arguments{type};
	my $content = $arguments{content};
	my $server  = $arguments{server} || undef;

	# Path to tollrestriction.xml
	my $tollxml = "/etc/asterisk/tollrestriction.xml";

	# Debug mode?
	my $debug = 1;

	&tollrest_debug("Tollrest macro called: type=$type; content=$content; server=$server\n");

	# Server ID must be valid.
	if (defined $server && length $server)
	{
		if ($server =~ /^\d+$/)
		{
			# Okay.
		}
		else
		{
			$server = undef;
		}
	}

	# $type will be one of:
	# extension = local extension
	# remote    = remote extension on another linked server
	# dialplan  = a normal dial plan
	#
	# $content  = the actual extension or dial plan string
	#
	# $server   = the server ID that the extension belongs to
	#             (or undef for local server)

	# Find out our local server ID.
	my $globals = "/etc/asterisk/fonality/globals.conf";
	my $localServer = undef;
	open (GLOBALS, $globals);
	while (<GLOBALS>)
	{
		if (/^IAXINFO\d?=s(\d{4,6}):([^;\s]+)/)
		{
			$localServer = $1;
			last;
		}
	}

	# If a server ID was passed, it might be from link.conf.
	my $fromLinkConf = 1; # Guilty until proven innocent

	# If not given a server, assume the local server.
	if (!defined $server || !length $server)
	{
		# This proves us innocent.
		$fromLinkConf = 0;

		# Use the local server.
		$server = $localServer;
	}

	# One last check if this is the local server or not.
	if ($server == $localServer) {
		$fromLinkConf = 0;
	}
	else
	{
		$fromLinkConf = 1;
	}

	# Get the caller's extension number.
	my $caller = 7002; #$AGI->get_variable('CALLERID(num)');

	&tollrest_debug ("Looking up group membership\n") if $debug;

	# We need to cache the p3.xml structure unless it changes.
	my @stat = stat("/etc/asterisk/hud3.0/p3.xml");
	if (!exists $AGI->{"tollrest_p3_lastmod"} || $AGI->{"tollrest_p3_lastmod"} != $stat[9])
	{
		# Look up their permission data and find out which groups they're in.
		my $perms_p3 = get_permissions();

		# Simplify the perms structure.
		my $perms = {};
		foreach my $toplevel (@{$perms_p3->{permissions}})
		{
			next unless exists $toplevel->{group};
			foreach my $group (keys %{$toplevel->{group}})
			{
				# Get the group's members.
				my $exts = '';
				if (ref($toplevel->{group}->{$group}->{members}) eq "ARRAY") {
					foreach my $mem (@{$toplevel->{group}->{$group}->{members}})
					{
						next unless $mem->{server} == $localServer;
						$exts .= ',' if length $exts > 0;
						$exts .= $mem->{ext};
					}
				}
				elsif (ref($toplevel->{group}->{$group}->{members}) eq "HASH") {
					$exts = $toplevel->{group}->{$group}->{members}->{ext};
				}

				# Map the extensions.
				for (split(/\,/, $exts)) {
					next unless length $_;
					$perms->{ext}->{"e$_"} = 1;
				}

				# Store the group info.
				$perms->{ext_groups}->{ "g$toplevel->{group}->{$group}->{id}" } = $exts;
			}
		}

		# Cache the perms.
		$AGI->{"tollrest_p3_cache"} = $perms;
		$AGI->{"tollrest_p3_lastmod"} = $stat[9];
	}
	$perms = $AGI->{"tollrest_p3_cache"};

	print Dumper($perms); use Data::Dumper;

	# Actually, first just verify if they belong to this server. If not, our
	# rules don't apply to them.
	if (!exists $perms->{ext}->{"e$caller"})
	{
		# They don't even belong to us!
		#$AGI->execute("SET VARIABLE TOLLRESTOK TRUE");
		&tollrest_debug ("Caller e$caller doesn't belong to us!");
		return;
	}

	# Now continue.
	my %inGroups = ();
	foreach my $grp (keys %{$perms->{ext_groups}})
	{
		&tollrest_debug ("Checking in group $grp\n") if $debug;
		my @es = split(/\,/, $perms->{ext_groups}->{$grp});
		my @extlist = ();

		# Get all the extensions in this group.
		for (my $ei = 0; $ei < scalar(@es); $ei++)
		{
			my $e = $es[$ei];
			
			&tollrest_debug ("\tGroup has ext $e\n") if $debug;
			# Is this a range?
			if ($e =~ /^r\((\d+)$/i && $es[$ei + 1] =~ /^\d+\)$/)
			{
				# Get the other extension.
				my ($lower) = $e =~ /^r\((\d+)$/i;
				my ($higher) = $es[$ei + 1] =~ /^(\d+)\)$/;
				$ei++;
				&tollrest_debug ("\t-> This is a range from $lower to $higher\n") if $debug;
				for (my $i = $lower; $i <= $higher; $i++)
				{
					&tollrest_debug ("\t-> Adding ext $i to the list to check\n") if $debug;
					push (@extlist, $i);
				}
			}
			else
			{
				&tollrest_debug ("\t-> Adding ext $e to the list to check\n") if $debug;
				push (@extlist, $e);
			}
		}

		# See if any of these extensions are the caller's.
		&tollrest_debug ("\tSeeing if any of this group's ext's are the caller's\n") if $debug;
		foreach my $num (@extlist)
		{
			&tollrest_debug ("\t\t-> Comparing to ext $num\n") if $debug;
			if ($num eq $caller)
			{
				&tollrest_debug ("\t\t-> We have a group membership!!\n") if $debug;
				# We belong to this group.
				my ($gid) = $grp =~ /^g(\d+)$/i;
				$inGroups{$gid} = 1;
				last;
			}
		}
	}

	# By default they can NOT dial this.
	my $can_dial = 0;

	# Read our XML file, if it exists.
	my $mostlyblank = 0;
	if (-f $tollxml)
	{
		open (READ, $tollxml);
		my @xml = <READ>;
		close (READ);
		chomp @xml;
		my $xml_data = join("\n",@xml);

		# Perlify it.
		my $toll = XMLin ($xml_data, ForceArray => 1);

		# Is it mostly blank?
		if (ref($toll) ne "HASH" || scalar(keys(%{$toll})) == 0)
		{
			# It's mostly blank. Stop processing things.
			$mostlyblank = 1;
			&tollrest_debug ("The tollrestriction.xml file exists, but is mostly blank, so can_dial = 1!\n");
		}

		&tollrest_debug ($xml_data . "\n" . $tollxml . "\n" . Dumper($toll) . "\n\ncaller=$caller\nserver=$server\ntype=$type\n")
		if $debug;
		use Data::Dumper;

		# Go through and see if their dialing request is acceptable.
		my @debug = ("Seeing if their dial request is acceptable.");
		foreach my $gid (keys %inGroups) {
			push (@debug, "\t-> Looking at the perms for group $gid");
			next unless exists $toll->{group}->{$gid};
			push (@debug, "\t-> Looping through the toll restriction items");
			foreach my $item (@{$toll->{group}->{$gid}->{item}})
			{
				my $item_type = $item->{type};
				my $item_content = $item->{content};
				push (@debug,"\t-> Found content=$item_content and type=$item_type");

				if (($type eq "extension" || $type eq "remote") && $item_type eq "linked_server")
				{
					# We're dialing an extension and this restriction is
					# about a server. Is this the server we want to dial?
					push (@debug,"\t-> Item is: extension on a server ($item_content <=> $server)");
					if ($item_content eq $server)
					{
						push (@debug,"\t-> can_dial = 1!");
						# Good deal.
						$can_dial = 1;
						last;
					}
				}
				elsif ($type eq "dialplan" && $item_type eq "dial_string")
				{
					# We're dialing a normal dial string and this restriction
					# is about a dial string. Is it what we want?
					push (@debug,"\t-> Item is: dialplan ($item_content <=> $content)");
					if ($item_content eq $content)
					{
						push (@debug, "\t-> can_dial = 1!");
						# Good deal.
						$can_dial = 1;
						last;
					}
				}
			}

			# Break if we've got our answer.
			last if $can_dial == 1;
		}
		&tollrest_debug (join("\n",@debug) . "\n") if $debug;
	}
	else
	{
		# Our XML file doesn't exist. This (probably) shouldn't happen,
		# so assume there was a major error and automatically allow all
		# requests to pass.
		$can_dial = 1;
		&tollrest_debug ("There is no tollrestriction.xml, so can_dial = 1!\n");
	}

	# was the XML mostly blank?
	if ($mostlyblank)
	{
		$can_dial = 1;
		&tollrest_debug ("tollrestriction.xml was mostly blank, so can_dial = 1!\n");
	}

	if (!defined $server)
	{
		# This shouldn't happen.
		$can_dial = 1;
	}

	# Return whether we're going to allow this or not.
	&tollrest_debug ("The verdict: can_dial = $can_dial\n\n\n");
	my $truefalse = $can_dial ? "TRUE" : "FALSE";

	# If we're saying "FALSE", but this was a check on a linked server,
	# flag it as "TRUE" and let Asterisk dig deeper and call us back for
	# the local server.
#	if ($truefalse eq "FALSE" && $fromLinkConf)
#	{
#		$truefalse = "TRUE";
#	}

	#$AGI->execute("SET VARIABLE TOLLRESTOK $truefalse");
}

# vim: ft=perl
